import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:flutter/foundation.dart';
import 'package:flutterx_http/flutterx_http.dart';

@immutable
class Request extends IRequest {
  final String method;
  final Uri url;
  final QueryParameters params;
  final HeadersMap headers;
  final Object? body;
  final Encoding? encoding;

  const Request({
    required this.method,
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  });

  const Request.head({
    required this.url,
    this.params = const {},
    this.headers = const {},
  })  : method = 'HEAD',
        body = null,
        encoding = null;

  const Request.get({
    required this.url,
    this.params = const {},
    this.headers = const {},
  })  : method = 'GET',
        body = null,
        encoding = null;

  const Request.post({
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  }) : method = 'POST';

  const Request.put({
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  }) : method = 'PUT';

  const Request.patch({
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  }) : method = 'PATCH';

  const Request.delete({
    required this.url,
    this.params = const {},
    this.headers = const {},
  })  : method = 'DELETE',
        body = null,
        encoding = null;

  @override
  XHttpRequest finalize() => XHttpRequest(
      method: method,
      url: url.appendQueryParameters(params),
      headers: {HttpHeaders.contentTypeHeader: 'application/json', ...headers},
      body: _encodeBody(body, encoding ?? utf8));

  static XHttpContent? _encodeBody(Object? body, Encoding encoding) => body is XHttpContent
      ? body
      : XHttpContent.fromBytes(Uint8List.fromList(encoding.encode(body is String ? body : jsonEncode(body))));
}
